@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # $Id: __init__.py 11669 2019-07-31 11:16:05Z Trista $
 #
-# Copyright (c) 2015 Nuwa Information Co., Ltd, All Rights Reserved.
+# Copyright (c) 2019 Nuwa Information Co., Ltd, All Rights Reserved.
 #
 # Licensed under the Proprietary License,
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,8 @@ import re
 
 import requests
 
+from Crypto.PublicKey import RSA
+
 from decorator import decorator
 from termcolor import colored, cprint
 from fabric import task
@@ -44,22 +46,59 @@ DEPLOY_TEST_SERVER_USER = os.environ.get(
 DEPLOY_TEST_SERVER_PASSWORD = os.environ.get(
     'DEPLOY_TEST_SERVER_PASSWORD', '@%)@5529')
 
-def getConnectKwargs(password=None):
+def getConnectKwargs(password=None, key=os.path.join("PrivateKey.key")):
     # put key
     if password:
         return {"password": password,}
     else:
-        if os.path.exists("PrivateKey.key"):
-            return {"key_filename": "PrivateKey.key",}
+        if os.path.exists(key):
+            return {"key_filename": key,}
         else:
             return None
+            
+def getKey(username, keyPath=os.path.join("Keys")):
+    if not os.path.exists(keyPath):
+        os.mkdir(keyPath)
+    userPath = os.path.join(keyPath, username) 
+    privateKeyPath = os.path.join(userPath, "PrivateKey.key")
+    publicKeyPath = os.path.join(userPath, "PublicKey.key")
+    
+    # check key
+    if os.path.isdir(userPath):
+        # get key
+        with open(publicKeyPath, 'r') as f:
+            publicKey = f.read()
+        with open(privateKeyPath, 'r') as f:
+            privateKey = f.read()
+    else:
+        # make key
+        os.mkdir(userPath)
+
+        key = RSA.generate(4096)
+        
+        publicKey = key.publickey().exportKey('OpenSSH').decode("utf-8")
+        privateKey = key.exportKey("PEM").decode("utf-8")
+        
+        with open(publicKeyPath, 'w') as f:
+            
+            f.write(publicKey)
+            
+        with open(privateKeyPath, 'w') as f:
+            privateKey = privateKey.replace(
+                "BEGIN PRIVATE KEY", "BEGIN RSA PRIVATE KEY")
+            privateKey = privateKey.replace(
+                "END PRIVATE KEY", "END RSA PRIVATE KEY")
+
+            f.write(privateKey)
+            
+            
+    return (publicKey, privateKey, publicKeyPath, privateKeyPath)
 
 @decorator
 def record(func, c, *args, **kwargs):
     rt = None
     try:
         rt = func(c, *args, **kwargs)
-        c.recordLogs()
     except Exception as e:
         cprint(
             colored(
@@ -68,8 +107,9 @@ def record(func, c, *args, **kwargs):
             )
         )
         c.addLog("ERROR: %s" % str(e))
-        c.recordLogs()
         raise
+    finally:
+        c.recordLogs()
     
     return rt
  
@@ -116,7 +156,7 @@ def recordLogs(self):
     if "key_filename" in self.connect_kwargs:
         from Crypto.PublicKey import RSA
 
-        with open("PrivateKey.key", "r") as f:
+        with open(self.connect_kwargs["key_filename"], "r") as f:
             key = RSA.import_key(f.read())
             
         publicKey = key.publickey().exportKey('OpenSSH').decode("utf-8")
